#******************************
# 
# Copyright (C) Oliver Scholl < tcltk@posteo.de >
##
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation; either version 2 of 
# the license, or any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# merchantability or fitness for a particular purpose. 
# See the GNU General Public License for more details.
#
#*************************************

package require msgcat

font create GlobSchrift -family Helvetica -size 10 -slant roman -weight normal
ttk::style theme use default
ttk::style configure TButton -font GlobSchrift
ttk::style configure TLabel -font GlobSchrift 
ttk::style configure TLabelFrame -font GlobSchrift 
ttk::style configure TMenubutton -font GlobSchrift 
ttk::style configure TRadiobutton -font GlobSchrift
ttk::style configure TEntry -font GlobSchrift 
ttk::style configure TCombobox -font GlobSchrift 
ttk::style configure TCheckbutton -font GlobSchrift
ttk::style configure TNotebook.Tab -font GlobSchrift 
ttk::style configure Treeview -font GlobSchrift 
ttk::style configure Treeview.Heading -font GlobSchrift 

option add *Dialog.msg.font GlobSchrift

########### GlobPrg ###########
#set GlobPrg(ProgrammName) "osTest"
#set GlobPrg(ProgrammVersion) "2018-xx-xx"
#set GlobPrg(ProgrammTitel) "Titel Test"
set GlobPrg(ProgrammOrdner) ".osProgramme"
set GlobPrg(ProgrammCopyright) "Copyright (C) Oliver Scholl\nwww.tcltk.info"

#set GlobPrg(Skriptname) [info script]
#if {[file type $GlobPrg(Skriptname)] == "link"} {set GlobPrg(Skriptname) [file readlink $GlobPrg(Skriptname)]}

set GlobPrg(Homeordner) [file join $env(HOME) $GlobPrg(ProgrammOrdner) $GlobPrg(ProgrammName)]
if {[file exist $GlobPrg(Homeordner)] == 0} {file mkdir $GlobPrg(Homeordner)}

set GlobPrg(Konfigurationsdatei) [file join $GlobPrg(Homeordner) Konf.txt] ; # Dateiname der Konfigurationsdatei

########### GlobKonf und GlobGui ###########

array set GlobKonf {} ; # Konfigurations-Variablen
array set GlobGui {} ; # GUI-Variablen

########### Sprache ###########

::msgcat::mclocale de
::msgcat::mcload [file join [file dirname $GlobPrg(Skriptname)]]


########### Beginn Prozeduren #########

proc MasterInfoAnzeigen {} {
	global GlobPrg
	set Titel [::msgcat::mc "Info"]
	set Text "$GlobPrg(ProgrammName) $GlobPrg(ProgrammVersion)\n\n$GlobPrg(ProgrammCopyright)"
	tk_messageBox -title $Titel -message $Text -type ok
}

proc MasterKonfigurationLaden {} {
	# laedt die Konfigurationsdaten
	global GlobPrg
	global GlobKonf
	source $GlobPrg(Konfigurationsdatei)
}

proc MasterKonfigurationSpeichern {} {
	# speichert die Konfigurationsdaten
	global GlobPrg
	global GlobKonf

	set f [open $GlobPrg(Konfigurationsdatei) w]
	puts $f "array set GlobKonf [list [array get GlobKonf]]"
	close $f
}

proc MasterProgrammBeenden {} {
	# Beendet das Programm
	global GlobKonf
	
	catch {ProgrammBeenden}
	
	# Fenstergroesse und -position
	set GlobKonf(FensterBreite) [winfo width .]
	set GlobKonf(FensterHoehe) [winfo height .]
	set GlobKonf(FensterX) [winfo x .]
	set GlobKonf(FensterY) [winfo y .]
	
	MasterKonfigurationSpeichern
	destroy .
	exit
}

proc MasterFensterGroessePositionSetzen {} {
	# Setzt die Fenstergroesse und -position beim Programmstart
	global GlobKonf

	set tmpBreite $GlobKonf(FensterBreite)
	set tmpHoehe $GlobKonf(FensterHoehe)
	set tmpX $GlobKonf(FensterX)
	set tmpY $GlobKonf(FensterY)
	wm geometry . ${tmpBreite}x${tmpHoehe}+${tmpX}+${tmpY}
}

proc MasterSpracheAendern {} {
	# Sprache aendern
	global GlobPrg
	global GlobKonf
	
	switch $GlobKonf(Sprache) {
		Deutsch {set GlobKonf(Sprache) Deutsch; set SpracheKurzform de}
		English {set GlobKonf(Sprache) English; set SpracheKurzform en}
		default {set GlobKonf(Sprache) English; set SpracheKurzform en}
	}
	::msgcat::mclocale $SpracheKurzform
	::msgcat::mcload [file join [file dirname $GlobPrg(Skriptname)]]

	.mbar entryconfigure 1 -label [::msgcat::mc "Datei"]
	.mbar entryconfigure 2 -label [::msgcat::mc "Einstellungen"]
	.mbar entryconfigure 3 -label [::msgcat::mc "Info"]

	.mbar.datei entryconfigure 0 -label [::msgcat::mc "Beenden"]

	.mbar.einstellung entryconfigure 0 -label [::msgcat::mc "Sprache"]
	.mbar.einstellung entryconfigure 1 -label [::msgcat::mc "Schriftgroesse"]

	catch {SpracheAendern}
	return
}

############### Hauptprogramm 

# Anfangswerte setzen, falls keine Konfigurationsdatei existiert


# Konfiguration laden, wenn Datei existiert
if {[file exist $GlobPrg(Konfigurationsdatei)] == 1} {
	MasterKonfigurationLaden
} else {
	# Initialte Fenstergröße und -position, Schriftgroesse, Sprache
	# wenn keine Konfigurationsdatei existiert
	set GlobKonf(FensterBreite) 800
	set GlobKonf(FensterHoehe) 600
	set GlobKonf(FensterX) 10
	set GlobKonf(FensterY) 10
	set GlobKonf(Schriftgroesse) 10
	set GlobKonf(Sprache) Deutsch
}

# GlobSchriftgroesse einstellen
set GlobGui(Schriftgroesse) $GlobKonf(Schriftgroesse)
font configure GlobSchrift -size $GlobGui(Schriftgroesse)

# Sprache einstellen
set GlobGui(Sprache) $GlobKonf(Sprache)

wm title . $GlobPrg(ProgrammTitel)
wm protocol . WM_DELETE_WINDOW {MasterProgrammBeenden}
MasterFensterGroessePositionSetzen

menu .mbar -font GlobSchrift
. configure -menu .mbar
.mbar add cascade -label [::msgcat::mc "Datei"] -menu .mbar.datei -underline 0
.mbar add cascade -label [::msgcat::mc "Einstellungen"] -menu .mbar.einstellung -underline 0
.mbar add command -label [::msgcat::mc "Info"] -underline 0 -command {MasterInfoAnzeigen}
menu .mbar.datei -tearoff 0 -font GlobSchrift
.mbar.datei add command -label [::msgcat::mc "Beenden"] -underline 0 -command {MasterProgrammBeenden}
menu .mbar.einstellung -tearoff 0 -font GlobSchrift
.mbar.einstellung add cascade -label [::msgcat::mc "Sprache"] -menu .mbar.einstellung.sprache -underline 0
menu .mbar.einstellung.sprache -tearoff 0 -font GlobSchrift
.mbar.einstellung.sprache add radiobutton -label "Deutsch" -variable GlobGui(Sprache) -value Deutsch -underline 0 -command {set GlobKonf(Sprache) $GlobGui(Sprache);MasterSpracheAendern}
.mbar.einstellung.sprache add radiobutton -label "English" -variable GlobGui(Sprache) -value English -underline 0 -command {set GlobKonf(Sprache) $GlobGui(Sprache);MasterSpracheAendern}
.mbar.einstellung add cascade -label [::msgcat::mc "Schriftgroesse"] -menu .mbar.einstellung.schriftgroesse -underline 0
menu .mbar.einstellung.schriftgroesse -tearoff 0 -font GlobSchrift
for {set i 6} {$i <= 14} {set i [expr $i+1]} {
	.mbar.einstellung.schriftgroesse add radiobutton -label $i -variable GlobGui(Schriftgroesse) -value $i -underline 0 -command {set GlobKonf(Schriftgroesse) $GlobGui(Schriftgroesse);font configure GlobSchrift -size $GlobGui(Schriftgroesse)}
}
for {set i 16} {$i <= 30} {set i [expr $i+2]} {
	.mbar.einstellung.schriftgroesse add radiobutton -label $i -variable GlobGui(Schriftgroesse) -value $i -underline 0 -command {set GlobKonf(Schriftgroesse) $GlobGui(Schriftgroesse);font configure GlobSchrift -size $GlobGui(Schriftgroesse)}
}

MasterSpracheAendern
