#!/usr/bin/env wish

#******************************
# 
# osFoto
# Version 2018-03-19
# Copyright (C) 2012-2017 Oliver Scholl < tcltk@posteo.de >
#
# This program is free software; you can redistribute it and/or 
# modify it under the terms of the GNU General Public License as 
# published by the Free Software Foundation; either version 2 of 
# the license, or any later version.
#
# This program is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# merchantability or fitness for a particular purpose. 
# See the GNU General Public License for more details.
#
#*************************************

proc SpracheAendern {} {
	.n tab .n.f5 -text [::msgcat::mc "Fotos importieren"]
	.n tab .n.f1 -text [::msgcat::mc "Dateiname mit Datum und Uhrzeit erzeugen"]
	.n tab .n.f3 -text [::msgcat::mc "Bildgroesse aendern"]
	.n.f5.lbZiel configure -text [::msgcat::mc "Ziel:"]
	.n.f5.btStart configure -text [::msgcat::mc "Start"]
	.n.f1.lbQuelle configure -text [::msgcat::mc "Quelle:"] 
	.n.f1.lbZiel configure -text [::msgcat::mc "Ziel:"] 
	.n.f1.lbText configure -text [::msgcat::mc "Text:"]
	.n.f1.cbDateinameHinzufuegen configure -text [::msgcat::mc "Bisherigen Dateiname hinzufuegen"]
	.n.f1.btStart configure -text [::msgcat::mc "Start"]
	.n.f3.lbQuelle configure -text [::msgcat::mc "Quelle:"]
	.n.f3.lbZiel configure -text [::msgcat::mc "Ziel:"] 
	.n.f3.lbKurzeSeite configure -text [::msgcat::mc "Kurze Seite (Pixel):"]
	.n.f3.lbAbzug configure -text [::msgcat::mc "Abzug (cm):"]
	.n.f3.lbDPI configure -text [::msgcat::mc "dpi:"]
	.n.f3.lbLangeSeite configure -text [::msgcat::mc "Lange Seite (Pixel):"]
	.n.f3.lbProzent configure -text [::msgcat::mc "Prozent:"]
	.n.f3.lbQualitaet configure -text [::msgcat::mc "Qualitaet (1-100):"]
	.n.f3.cbVergroessern configure -text [::msgcat::mc "Vergroesserung zulassen"]
	.n.f3.btStart configure -text [::msgcat::mc "Start"]
	.f4.lbStatus configure -text [::msgcat::mc "Status:"]
	update idletasks
}

proc AbzugsgroesseBerechnen {Zentimeter DPI} {
	# Berechnet die Anzahl der Pixel für die kurze Bildseite,
	# wenn die kurze Seite beim Abzug x Zentimeter lang sein soll
	# und der Abzug mit x dpi erfolgt.
	
	# Abzüge erstellen: Maßgeblich ist die kurze Bildseite.
	# 1 inch = 2,54 cm / Druckerauflösung: 300 dpi
	# Formel: KurzeSeite = Format in cm / 2.54 * 300
	# 9er-Format: 1062 Pixel
	# 10er-Format: 1181 Pixel
	# 13er-Format: 1535 Pixel

	if {$Zentimeter == ""} {set Zentimeter 0}
	if {$DPI == ""} {set DPI 0}
	
	if {$DPI != 0} {
		set Wert [expr int($Zentimeter/2.54*$DPI)]
	} else {
		set Wert 0
	}
	return $Wert
}

proc FotoImportieren {Verzeichnis} {
	# Importiert die Fotos von der Kamera

	global Fortschritt ;# aktualisiert den Statustext im Dialog
	
	set Fortschritt [::msgcat::mc "in Arbeit"]
	update idletasks
	
	cd $Verzeichnis
	catch {exec gphoto2 --auto-detect}
	#exec gphoto2 --list-files
	catch {exec gphoto2 --get-all-files}

	set Fortschritt [::msgcat::mc "Fertig!"]
	update idletasks
}

proc FotoGroesseAendern {QuelleVerzeichnis ZielVerzeichnis Prozent {LangeSeite 0} {KurzeSeite 0} {Qualitaet 100} {Vergroessern 0}} {
	# Ändert die Größe oder die Qualität eines Fotos
	# Prozent (0-100): neue Bildgröße in Prozent. Das Seitenverhältnis wird beibehalten.
	# LangeSeite: neue Länge (in Pixel) der langen Bildseite. Das Seitenverhältnis wird beibehalten, wenn der Wert für die KurzeSeite = 0 ist.
	# KurzeSeite: neue Länge (in Pixel) der kurzen Bildseite. Das Seitenverhältnis wird beibehalten, wenn der Wert für die LangeSeite = 0 ist.
	# Wenn LangeSeite > 0 und KurzeSeite > 0, dann wird das Bild genau auf diese Größe gebracht. Dabei wird das Seitenverhältnis auch geändert.
	# Qualitaet (0-100): neue Bildqualität (100 = beste Qualität).
	# Vergroessern (Boolean): Standardmäßig wird das Bild nur verkleinert. Wenn es auch vergrößert werden darf, muss das Flag auf 1 gesetzt werden.

	global Fortschritt ;# aktualisiert den Statustext im Dialog

	set Fortschritt [::msgcat::mc "in Arbeit"]
	update idletasks
	
	if {$Prozent == ""} {set Prozent 0}
	if {$LangeSeite == ""} {set LangeSeite 0}
	if {$KurzeSeite == ""} {set KurzeSeite 0}
	if {$Qualitaet == ""} {set Qualitaet 100}
	if {$Vergroessern == ""} {set Vergroessern 0}

	if {$QuelleVerzeichnis == ""} {return}
	if {$ZielVerzeichnis == ""} {return}
	if {$QuelleVerzeichnis == $ZielVerzeichnis} {return}
	if {($Prozent == 0) && ($LangeSeite == 0) && ($KurzeSeite == 0)} {return}
	
	if {$Qualitaet > 100} {set Qualitaet 100}
	
	set Liste [glob -directory $QuelleVerzeichnis -nocomplain *.{jpg,jpeg,JPG,JPEG}]
	foreach Datei $Liste {
		set Fortschritt [::msgcat::mc "%s" $Datei]
		update idletasks
		set Dateiname [file tail $Datei]
		set NeuDatei [file join $ZielVerzeichnis $Dateiname]
		#puts $NeuDatei
		set OriginalBreite [exec identify -format "%w" $Datei]
		set OriginalHoehe [exec identify -format "%h" $Datei]
		#puts "Format: $OriginalBreite x $OriginalHoehe"
		
		if {$OriginalBreite >= $OriginalHoehe} {
			set OriginalLangeSeite $OriginalBreite
			set OriginalKurzeSeite $OriginalHoehe
		} else {
			set OriginalLangeSeite $OriginalHoehe
			set OriginalKurzeSeite $OriginalBreite
		}
		
		# Default-Werte setzen
		set NeuLangeSeite $OriginalLangeSeite
		set NeuKurzeSeite $OriginalKurzeSeite

		if {($Prozent > 0) && ($LangeSeite == 0) && ($KurzeSeite == 0)} {
			set NeuLangeSeite [expr int(floor(1.0*$OriginalLangeSeite*$Prozent/100))]
			set NeuKurzeSeite [expr int(floor(1.0*$OriginalKurzeSeite*$Prozent/100))]
		}
		if {($Prozent == 0) && ($LangeSeite > 0) && ($KurzeSeite == 0)} {
			set NeuLangeSeite [expr int(floor($LangeSeite))]
			set NeuProzent [expr 1.0*$NeuLangeSeite/$OriginalLangeSeite*100]
			set NeuKurzeSeite [expr int(floor(1.0*$OriginalKurzeSeite*$NeuProzent/100))]
		}
		if {($Prozent == 0) && ($LangeSeite == 0) && ($KurzeSeite > 0)} {
			set NeuKurzeSeite [expr int(floor($KurzeSeite))]
			set NeuProzent [expr 1.0*$NeuKurzeSeite/$OriginalKurzeSeite*100]
			set NeuLangeSeite [expr int(floor(1.0*$OriginalLangeSeite*$NeuProzent/100))]
		}
		if {($Prozent == 0) && ($LangeSeite > 0) && ($KurzeSeite > 0)} {
			set NeuLangeSeite [expr int(floor($LangeSeite))]
			set NeuKurzeSeite [expr int(floor($KurzeSeite))]
		}
		#puts "NL=$NeuLangeSeite OL=$OriginalLangeSeite NK=$NeuKurzeSeite OK=$OriginalKurzeSeite"
		if {($Vergroessern == 1) || (($NeuLangeSeite<$OriginalLangeSeite) && ($NeuKurzeSeite<$OriginalKurzeSeite))} {
			if {$OriginalBreite >= $OriginalHoehe} {
				set NeuBreite $NeuLangeSeite
				set NeuHoehe $NeuKurzeSeite
			} else {
				set NeuBreite $NeuKurzeSeite
				set NeuHoehe $NeuLangeSeite
			}
			exec convert -resize ${NeuBreite}x$NeuHoehe\! -quality $Qualitaet $Datei $NeuDatei
			set OriginalLangeSeite [exec identify -format "%w" $NeuDatei]
			set OriginalKurzeSeite [exec identify -format "%h" $NeuDatei]
			#puts "Neues Format: $OriginalLangeSeite x $OriginalKurzeSeite"
		}
	}

	set Fortschritt [::msgcat::mc "Fertig!"]
	update idletasks
	return 1
}

proc FotoUmbenennen {QuelleVerzeichnis ZielVerzeichnis DateinameHinzufuegen {Text ""}} {
	# sucht alle jpg-Dateien im aktuellen Ordner
	# Ersetzt den Dateinamen durch Datum-Uhrzeit-Text

	global Fortschritt ;# aktualisiert den Statustext im Dialog

	set Fortschritt [::msgcat::mc "in Arbeit"]
	update idletasks
	
	if {$QuelleVerzeichnis == ""} {return}
	if {$ZielVerzeichnis == ""} {return}
	if {$QuelleVerzeichnis == $ZielVerzeichnis} {return}
	
	set Liste [glob -directory $QuelleVerzeichnis -nocomplain *.{jpg,jpeg,JPG,JPEG}]
	foreach Datei $Liste {
		#puts $Datei
		set Fortschritt [::msgcat::mc "%s" $Datei]
		update idletasks

		set a [::jpeg::getExif $Datei]

		# Listet alle Exif-Einträge auf
		#dict for {id info} $a {
		#	puts "$id:$info"
		#}

		#puts [dict get $a DateTimeOriginal]
		#puts [dict get $a ExifImageWidth]
		#puts [dict get $a ExifImageHeight]
		#puts [dict get $a Orientation]

		# Datum und Zeit extrahieren
		set DatumZeit "9999:99:99 00:00:00"
		catch {set DatumZeit [dict get $a DateTime]}
		catch {set DatumZeit [dict get $a DateTimeOriginal]}
		
		set Felder [split $DatumZeit " "]
		lassign $Felder Datum Zeit
		#puts "$Datum $Zeit"

		# Datum aufteilen in Jahr, Monat, Tag
		set Felder [split $Datum ":"]
		lassign $Felder Jahr Monat Tag
		#puts "$Jahr-$Monat-$Tag"

		# Zeit aufteilen in Stunde, Minute, Sekunde
		set Felder [split $Zeit ":"]
		lassign $Felder Stunde Minute Sekunde
		#puts "$Stunde:$Minute:$Sekunde"

		set Text [regsub -all " " $Text "_"]
		if {($Text ne "") && ([string index $Text 0] ne "-")} {
			set Text "-$Text"
		}
		
		if {$DateinameHinzufuegen == 1} {
			set tmp [file tail [file rootname $Datei]]
			set Text2 "$Text-$tmp" 
		} else {
			set Text2 $Text
		}
		
		# Liste aller Dateien, die denselben Dateinamen (ohne Endung) haben
		set tmpDateiname [file tail [file rootname $Datei]]
		set Liste2 [glob -directory $QuelleVerzeichnis -nocomplain $tmpDateiname.*]
		
		set Zaehler 0 ; # wenn Bilder mit gleichem Zeitstempel entstehen, wird ein Zähler im Dateiname ergänzt
		set Fertig 0 ; # bleibt solange 0, bis das Bild kopiert wurde 
		while {$Fertig == 0} {
			set Dateiname "$Jahr-$Monat-$Tag-${Stunde}${Minute}${Sekunde}${Text2}"
			if {$Zaehler > 0} {
				set Dateiname "$Dateiname${Zaehler}"
			}
			set Dateiname1 [file join $ZielVerzeichnis $Dateiname[file extension $Datei]]
			if {[glob -nocomplain -types f $Dateiname1] eq ""} {			
				# die Datei existiert nicht, deshalb kopieren
				file copy $Datei $Dateiname1
				
				# Alle Dateien, die denselben Dateinamen mit anderer Endung haben, ebenfalls kopieren und umbenennen
				foreach Datei2 $Liste2 {
					if {[file extension $Datei2] != [file extension $Datei]} {
						set Dateiname2 [file join $ZielVerzeichnis $Dateiname[file extension $Datei2]]
						file copy $Datei2 $Dateiname2
					}
				}
				set Fertig 1
			} else {
				# die Datei gibt es schon, Zähler um 1 erhöhen
				incr Zaehler
			}
		}
	}
	set Fortschritt [::msgcat::mc "Fertig!"]
	update idletasks
}

# ***** Hauptprogramm *****

set GlobPrg(Skriptname) [info script]
if {[file type $GlobPrg(Skriptname)] == "link"} {set GlobPrg(Skriptname) [file readlink $GlobPrg(Skriptname)]}
set GlobPrg(ProgrammName) "osFoto"
set GlobPrg(ProgrammVersion) "2018-03-19"
set GlobPrg(ProgrammTitel) "Foto"
source [file join [file dirname $GlobPrg(Skriptname)] master.tcl]
wm title . $GlobPrg(ProgrammTitel)

if {[file exist $GlobPrg(Konfigurationsdatei)] == 0} {
	# falls es noch keine Konfigurationsdatei gibt
	set GlobKonf(ImportVerzeichnis) $env(HOME)
	set GlobKonf(DateinameQuelleVerzeichnis) $env(HOME)
	set GlobKonf(DateinameZielVerzeichnis) $env(HOME)
	set GlobKonf(BildQuelleVerzeichnis) $env(HOME)
	set GlobKonf(BildZielVerzeichnis) $env(HOME)
}

source [file join [file dirname $GlobPrg(Skriptname)] exif.tcl]
source [file join [file dirname $GlobPrg(Skriptname)] jpeg.tcl]

set Text ""
set DateinameHinzufuegen 0
set KurzeSeite ""
set Abzug ""
set DPI 300
set LangeSeite ""
set Prozent ""
set Qualitaet 100
set Vergroessern 0
set Fortschritt ""

ttk::notebook .n

ttk::frame .n.f5
.n add .n.f5 -text [::msgcat::mc "Fotos importieren"]
ttk::label .n.f5.lbZiel -text [::msgcat::mc "Ziel:"] 
ttk::label .n.f5.lbZielVerzeichnis -textvariable GlobKonf(ImportVerzeichnis)
ttk::button .n.f5.btZielWaehlen -text "..." -command {set GlobKonf(ImportVerzeichnis) [tk_chooseDirectory -initialdir $env(HOME)]}
ttk::button .n.f5.btStart -text [::msgcat::mc "Start"] -command {FotoImportieren $GlobKonf(ImportVerzeichnis)}
pack .n -fill both -expand 1
grid .n.f5.lbZiel -row 1 -column 0 -sticky w -padx 3 -pady 3
grid .n.f5.lbZielVerzeichnis -row 1 -column 1 -sticky w -padx 3 -pady 3
grid .n.f5.btZielWaehlen -row 1 -column 2 -sticky w -padx 3 -pady 3
grid .n.f5.btStart -row 2 -column 0 -sticky w -padx 3 -pady 3
bind .n.f5.btStart <KeyPress-Return> ".f5.btStart invoke"

ttk::frame .n.f1
.n add .n.f1 -text [::msgcat::mc "Dateiname mit Datum und Uhrzeit erzeugen"]
ttk::label .n.f1.lbQuelle -text [::msgcat::mc "Quelle:"] 
ttk::label .n.f1.lbQuelleVerzeichnis -textvariable GlobKonf(DateinameQuelleVerzeichnis)
ttk::button .n.f1.btQuelleWaehlen -text "..." -command {set GlobKonf(DateinameQuelleVerzeichnis) [tk_chooseDirectory -initialdir $env(HOME)]}
ttk::label .n.f1.lbZiel -text [::msgcat::mc "Ziel:"] 
ttk::label .n.f1.lbZielVerzeichnis -textvariable GlobKonf(DateinameZielVerzeichnis)
ttk::button .n.f1.btZielWaehlen -text "..." -command {set GlobKonf(DateinameZielVerzeichnis) [tk_chooseDirectory -initialdir $env(HOME)]}
ttk::label .n.f1.lbText -text [::msgcat::mc "Text:"]
ttk::entry .n.f1.enText -width 20 -textvariable Text -font GlobSchrift
ttk::checkbutton .n.f1.cbDateinameHinzufuegen -text [::msgcat::mc "Bisherigen Dateiname hinzufuegen"] -variable DateinameHinzufuegen
ttk::button .n.f1.btStart -text [::msgcat::mc "Start"] -command {FotoUmbenennen $GlobKonf(DateinameQuelleVerzeichnis) $GlobKonf(DateinameZielVerzeichnis) $DateinameHinzufuegen $Text }
grid .n.f1.lbQuelle -row 0 -column 0 -sticky w -padx 3 -pady 3
grid .n.f1.lbQuelleVerzeichnis -row 0 -column 1 -sticky w -padx 3 -pady 3
grid .n.f1.btQuelleWaehlen -row 0 -column 2 -sticky w -padx 3 -pady 3
grid .n.f1.lbZiel -row 1 -column 0 -sticky w -padx 3 -pady 3
grid .n.f1.lbZielVerzeichnis -row 1 -column 1 -sticky w -padx 3 -pady 3
grid .n.f1.btZielWaehlen -row 1 -column 2 -sticky w -padx 3 -pady 3
grid .n.f1.lbText -row 2 -column 0 -sticky w -padx 3 -pady 3
grid .n.f1.enText -row 2 -column 1 -sticky we -padx 3 -pady 3
grid .n.f1.cbDateinameHinzufuegen -row 3 -column 0 -sticky w -padx 3 -pady 3 -columnspan 2
grid .n.f1.btStart -row 4 -column 0 -sticky w -padx 3 -pady 3
bind .n.f1.btStart <KeyPress-Return> ".n.f1.btStart invoke"

ttk::frame .n.f3
.n add .n.f3 -text [::msgcat::mc "Bildgroesse aendern"]
ttk::label .n.f3.lbQuelle -text [::msgcat::mc "Quelle:"] 
ttk::label .n.f3.lbQuelleVerzeichnis -textvariable GlobKonf(BildQuelleVerzeichnis)
ttk::button .n.f3.btQuelleWaehlen -text "..." -command {set GlobKonf(BildQuelleVerzeichnis) [tk_chooseDirectory -initialdir $env(HOME)]}
ttk::label .n.f3.lbZiel -text [::msgcat::mc "Ziel:"] 
ttk::label .n.f3.lbZielVerzeichnis -textvariable GlobKonf(BildZielVerzeichnis)
ttk::button .n.f3.btZielWaehlen -text "..." -command {set GlobKonf(BildZielVerzeichnis) [tk_chooseDirectory -initialdir $env(HOME)]}
ttk::label .n.f3.lbKurzeSeite -text [::msgcat::mc "Kurze Seite (Pixel):"]
ttk::entry .n.f3.enKurzeSeite -width 5 -textvariable KurzeSeite -font GlobSchrift
ttk::label .n.f3.lbAbzug -text [::msgcat::mc "Abzug (cm):"]
ttk::entry .n.f3.enAbzug -width 5 -textvariable Abzug -font GlobSchrift
ttk::label .n.f3.lbDPI -text [::msgcat::mc "dpi:"]
ttk::entry .n.f3.enDPI -width 5 -textvariable DPI -font GlobSchrift
ttk::label .n.f3.lbLangeSeite -text [::msgcat::mc "Lange Seite (Pixel):"]
ttk::entry .n.f3.enLangeSeite -width 5 -textvariable LangeSeite -font GlobSchrift
ttk::label .n.f3.lbProzent -text [::msgcat::mc "Prozent:"]
ttk::entry .n.f3.enProzent -width 5 -textvariable Prozent -font GlobSchrift
ttk::label .n.f3.lbQualitaet -text [::msgcat::mc "Qualitaet (1-100):"]
ttk::entry .n.f3.enQualitaet -width 5 -textvariable Qualitaet -font GlobSchrift
ttk::checkbutton .n.f3.cbVergroessern -text [::msgcat::mc "Vergroesserung zulassen"] -variable Vergroessern 
ttk::button .n.f3.btStart -text [::msgcat::mc "Start"] -command {FotoGroesseAendern $GlobKonf(BildQuelleVerzeichnis) $GlobKonf(BildZielVerzeichnis) $Prozent $LangeSeite $KurzeSeite $Qualitaet $Vergroessern}

grid .n.f3.lbQuelle -row 0 -column 0 -sticky w -padx 3 -pady 3
grid .n.f3.lbQuelleVerzeichnis -row 0 -column 1 -sticky w -padx 3 -pady 3
grid .n.f3.btQuelleWaehlen -row 0 -column 2 -sticky w -padx 3 -pady 3
grid .n.f3.lbZiel -row 1 -column 0 -sticky w -padx 3 -pady 3
grid .n.f3.lbZielVerzeichnis -row 1 -column 1 -sticky w -padx 3 -pady 3
grid .n.f3.btZielWaehlen -row 1 -column 2 -sticky w -padx 3 -pady 3
grid .n.f3.lbKurzeSeite -row 2 -column 0 -sticky w -padx 3 -pady 3
grid .n.f3.enKurzeSeite -row 2 -column 1 -sticky w -padx 3 -pady 3
grid .n.f3.lbAbzug -row 2 -column 2 -sticky w -padx 3 -pady 3
grid .n.f3.enAbzug -row 2 -column 3 -sticky w -padx 3 -pady 3
grid .n.f3.lbDPI -row 2 -column 4 -sticky w -padx 3 -pady 3
grid .n.f3.enDPI -row 2 -column 5 -sticky w -padx 3 -pady 3
grid .n.f3.lbLangeSeite -row 3 -column 0 -sticky w -padx 3 -pady 3
grid .n.f3.enLangeSeite -row 3 -column 1 -sticky w -padx 3 -pady 3
grid .n.f3.lbProzent -row 4 -column 0 -sticky w -padx 3 -pady 3
grid .n.f3.enProzent -row 4 -column 1 -sticky w -padx 3 -pady 3
grid .n.f3.lbQualitaet -row 5 -column 0 -sticky w -padx 3 -pady 3
grid .n.f3.enQualitaet -row 5 -column 1 -sticky w -padx 3 -pady 3
grid .n.f3.cbVergroessern -row 6 -column 0 -columnspan 6 -sticky w -padx 3 -pady 3
grid .n.f3.btStart -row 7 -column 0 -sticky w -padx 3 -pady 3
bind .n.f3.btStart <KeyPress-Return> ".n.f3.btStart invoke"
bind .n.f3.enAbzug <FocusOut> {set KurzeSeite [AbzugsgroesseBerechnen $Abzug $DPI]}
bind .n.f3.enDPI <FocusOut> {set KurzeSeite [AbzugsgroesseBerechnen $Abzug $DPI]}

.n select .n.f5

ttk::frame .f4
ttk::label .f4.lbStatus -text [::msgcat::mc "Status:"]
ttk::label .f4.lbFortschritt -textvariable Fortschritt

ttk::frame .sep4 -height 2 -borderwidth 1 -relief sunken
pack .sep4 -side top -fill x -pady 3

pack .f4 -side top -anchor w
grid .f4.lbStatus -row 0 -column 0 -sticky w -padx 3 -pady 3
grid .f4.lbFortschritt -row 0 -column 1 -sticky w -padx 3 -pady 3

